;         systemjob_asm                           06.05.1994 Jo

          errors    scr
          data      $100

version   equ       '1.04'    ; Version

;+++
; SYSTEMJOB misst die Hardware Geschwindigkeit des QL/Goldcard/Supergoldcard/
; QXL/Atari (wenn sonst nix luft), oder die Systembelastung (wenn weitere
; Jobs oder Interruptroutinen laufen).
;
; Systemjob startet 2 Interrupt Routinen:
; 1. Eine Scheduler Loop Routine zum Zhlen
; 2. Eine Polled Loop Routine zum Rcksetzen des Zhlers.
; Die Scheduler Routine zhlt aufwrts, so schnell sie kann. Nach genau einer
; Sekunde, gemessen mit der Polled Routine, wird das Ergebnis zwischenge-
; speichert und mit dem SYSTEMJOB asynchon ca. 1 mal pro Sekunde angezeigt.
; Die angezeigte Zahl ist also die "Anzahl der Scheduler Loop Umlufe pro
; Sekunde", oder die "Frequenz des Schedulers in Hz" (was das gleiche ist).
;
; Wenn keine weiteren Jobs oder Routinen laufen, ist die Frequenz des
; Schedulers nur abhngig von der verwendeten Hardware. Mit zunehmender
; Systembelastung geht die Frequenz immer weiter runter. Zum Beispiel
; durch Bewegen der Maus, wenn ptr_gen geladen ist. Oder durch einen der
; zahlreichen, schlecht geschiebenen Bildschirmschoner (!!).
;
; SYSTEMJOB luft ohne alles. Pointerumgebung nicht ntig! Da SYSTEMJOB die
; meiste Zeit suspendiert ist (von 1000 ms 980 ms inaktiv), ist der Messfehler
; durch den Job selber vernachlssigbar klein.
;
; Wenn der Button Frame ( QPAC2 ) vorhanden ist, wird er benutzt. (Ab V1.04)
; Die Position des Job-Fensters ohne Button Frame kann konfiguriert werden.
; Systemjob ist romfhig und verndert sich NICHT zur Laufzeit. Es gibt
; somit auch keine Cache-Probleme.
;
; Zum Testen der QL-Hardware ohne laufendes Basic: EXEC_W "flp1_SYSTEMJOB"
; Anschlieend ist ein Reset ntig.
; Zum Testen der QL-Hardware mit laufendem Basic: EXEC "flp1_SYSTEMJOB"
; Der Job darf trotz der gestarteten Interruptroutinen ganz normal mit RJOB,
; QPAC2 oder Cueshell gelscht werden (siehe remvjob_asm).
;
; 640k-QL ohne Basic, intern erweitert:             1020 Hz
; 640k-QL mit Basic+ptr_gen+wman, intern erweitert:  640 Hz
;
; Copyright: Jo 1992-94
; Das Programm ist frei. Bitte geben Sie es aber nur in unvernderter Form
; weiter. Danke.
; Jochen Hassler, Heilbronnerstr.7, 73728 Esslingen, 0711/3160212
;---

; rel (a3) oder (a5)
svbase    equ       $1C       ; long SV_BASE $28000 oder so
chid      equ       $20       ; long scr channel ID
pkilc     equ       $24       ; long $18 + $0C reserved !!!
sekunde   equ       $28       ; word sec timer poll
timer     equ       $2A       ; word timer register scheduler loop
count     equ       $2C       ; word timer count
hend      equ       $40       ; Lnge heap

          bra.s     start
          dc.b      '0605'
          dc.w      $4AFB
          dc.w      10
          dc.b      'System Job V'
          dc.l      version
          dc.b      ' 1994 Jo',$0A
start
          lea       0(a6,a4.l),a6       ; Pointer to data space
          moveq     #hend,d1            ; $40 bytes
          moveq     #-1,d2              ; me
          moveq     #$18,d0   ; MT_ALCHP
          trap      #1                  ; -> a0 base address of area
          tst.l     d0
            bne     err_kill
          movea.l   a0,a5

          moveq     #0,d0     ; MT_INF
          trap      #1
          move.l    a0,svbase(a5)
          movea.l   a6,a2
          lea       scrdef(pc),a1
          move.l    (a1)+,(a2)+         ; Rand,Breite,Papier,Tinte
          move.l    (a1),d1             ; dx/dy fr use_butn
          move.l    (a1)+,(a2)+         ; dx/dy
          move.l    (a1)+,(a2)+         ; x/y
          bsr       use_butn
            bne.s   nix_butn
          move.l    d1,8(a6)            ; new x/y in button frame
nix_butn
;         lea       scrdef(pc),a1
          movea.l   a6,a1               ; scrdef
          movea.w   $C8,a2    ; UT_SCR
          jsr       (a2)
          tst.l     d0
            bne.s   err_kill
          move.l    a0,chid(a5)

          lea       $18(a5),a2
          st        (a2)      ;testflag
          lea       killcode(pc),a1
          move.l    a2,$04-$10(a5)      ; DRIVR
          move.l    a1,$0C(a2)

          lea       $08(a5),a0          ; address of link
          lea       polltask(pc),a2
          move.l    a2,4(a0)            ; polled routine
          moveq     #$1C,d0   ; MT_LPOLL
          trap      #1

          lea       $10(a5),a0          ; address of link
          lea       schdtask(pc),a2
          move.l    a2,4(a0)            ; scheduler routine
          moveq     #$1E,d0   ; MT_LSCHD
          trap      #1

          moveq     #-1,d1              ; me
          moveq     #1,d2               ; Prioritt 1
          moveq     #$0B,d0   ; MT_PRIOR
          trap      #1
; -------------------------------------- Ende Ini, Start Jobloop
jobloop
          moveq     #-1,d1              ; me
          moveq     #49,d3              ; 49*20 ms = 1s - 20ms
          suba.l    a1,a1               ; no flag
          moveq     #$08,d0   ; MT_SUSJB
          trap      #1
          tst.l     d0
            bne.s   err_kill

          movea.l   chid(a5),a0
          moveq     #-1,d3
          moveq     #$20,d0   ; SD_CLEAR
          trap      #3
          tst.l     d0
            bne.s   err_kill

          moveq     #3,d1               ; x-pix
          moveq     #1,d2               ; y-pix
          moveq     #$17,d0   ; SD_PIXP
          trap      #3

          move.w    count(a5),d1
sendint   movea.w   $CE,a2    ; UT_MINT
          jsr       (a2)
          tst.l     d0
            beq.s   jobloop
; -------------------------------------- Ende Jobloop
err_kill
          move.l    d0,d3
          move.w    #-1,d1              ; me
          moveq     #$05,d0   ; MT_FRJOB
          trap      #1
err_ret   rts

scrdef    dc.b      4,1,7,0             ; rand,breite,paper,ink
          dc.w      44,14               ; dx, dy
xfdef     dc.w      0                   ; x position
yfdef     dc.w      2                   ; y position

;+++
; scheduler routine zum Zhlen
;         a3 c  p   pointer to dummy ddd
;---
schdtask
          addq.w    #1,timer(a3)
          rts

;+++
; polled task zum Rcksetzen
;         assert    sekunde+4,timer+2,count
;---
polltask
          lea       sekunde(a3),a1
          subq.w    #1,(a1)
            bgt.s   p_ret
          move.w    #50,(a1)+           ; sekundenzhler laden
          moveq     #0,d1
          move.w    (a1),d1             ; timer lesen
          move.l    d1,(a1)             ; timer rcksetzen und wert merken
p_ret     rts

;+++
;         d1      s
;         d2    p
;         d3      s
;         d4-d7     all preserved
;         a0 c    s pointer to heap header
;         a1 c    s pointer to killcode
;         a2    p
;         a3 c  p   drivr - $18
;---
killcode
          movem.l   d1-d3/a0-a3,-(a7)
          move.l    a0,d1               ; heap to free
          lea       $08(a3),a0          ; address of poll link
          tst.l     4(a0)
            beq.s   nopoll
          moveq     #$1D,d0   ; MT_RPOLL
          trap      #1
nopoll
          lea       $10(a3),a0          ; address of scheduler link
          tst.l     4(a0)
            beq.s   noschd
          moveq     #$1F,d0   ; MT_RSCHD
          trap      #1
noschd
          movea.l   d1,a0
          movea.w   $C2,a1    ; MM_RECHP
          jsr       (a1)
          movem.l   (a7)+,d1-d3/a0-a3
          rts

; ----------------------------- Use Button Frame
use_butn
L01EE   movem.l d2-d3/a0-a2,-(a7)
        moveq   #$00,d3
;        bra.s   L01FC
;        movem.l d2-d3/a0-a2,-(a7)
;        moveq   #-$01,d3
L01FC   move.l  d1,d2
        moveq   #-$01,d1
        lea     L0230(pc),a0
        moveq   #$28,d0
        jsr     L025E(pc)
        tst.l   d0
          beq.s L0210
        moveq   #-$01,d2
L0210   move.l  d2,d1
        movem.l (a7)+,d2-d3/a0-a2
        tst.l   d0
        rts
; 
;        movem.l d0-d3/a0-a2,-(a7)
;        moveq   #-$01,d1
;        lea     L0230(pc),a0
;        moveq   #$29,d0
;        jsr     L025E(pc)
;        movem.l (a7)+,d0-d3/a0-a2
;        rts
; 
L0230   dc.w    $000C
        dc.b    'Button Frame'

L025E   move.l  a4,-(a7)
        move.l  d0,-(a7)
        moveq   #$08,d0
        bsr.s   L0276
          bne.s L0272
        move.l  (a7)+,d0
        jsr     (a4)
L026C   movea.l (a7)+,a4
        tst.l   d0
        rts
; 
L0272   addq.l  #$04,a7
        bra.s   L026C
L0276   movem.l d1-d3/a0,-(a7)
        move.w  d0,d3
        moveq   #$00,d0         ; MT_INF
        trap    #$01
        move.l  $00B8(a0),d1    ; SYS_THGL
          beq.s L0290
        movea.l d1,a0
L0288   move.l  (a0),d1
          beq.s L0294
        movea.l d1,a0
        bra.s   L0288
L0290   moveq   #-$13,d0        ; ERR_NI
        bra.s   L02A6
L0294   movea.l $0010(a0),a0
        cmpi.l  #$FFFFFFFF,$0004(a0)
          bne.s L0290
        movea.l $00(a0,d3.w),a4
L02A6   movem.l (a7)+,d1-d3/a0
        tst.l   d0
        rts
; 
; ++++++++++++++++++++++++++++++++++++++++
config
          dc.b      '<<QCFX>>01'        ; Flag,level
          dc.w      10
          dc.b      'System Job'        ; Name
          dc.w      4
          dc.l      version             ; Version
; X-Position
          dc.b      10                  ; Item type: word
          dc.b      0                   ; Item selection Key
          dc.w      xfdef-*             ; Pointer to item
          dc.w      0                   ;            pre-proc
          dc.w      0                   ;            post-proc
          dc.w      text7-*             ;            description
          dc.w      attr7-*             ;            attributes
; Y-Position
          dc.b      10                  ; Item type: word
          dc.b      0                   ; Item selection Key
          dc.w      yfdef-*             ; Pointer to item
          dc.w      0                   ;            pre-proc
          dc.w      0                   ;            post-proc
          dc.w      text8-*             ;            description
          dc.w      attr8-*             ;            attributes
          dc.w      -1                  ; End flag
;postr
;          moveq     #$00,d1
;          move.b    fendef(pc),d0
;            beq.s   postrts
;          moveq     #$10,d1
;postrts   moveq     #$00,d0
;          rts

text7     dc.w      38
          dc.b      'X-Position des Fensters (0 ist links) '
attr7     dc.w      0,511-46

text8     dc.w      38
          dc.b      'Y-Position des Fensters (0 ist oben)  '
attr8     dc.w      0,255-14

          dc.w      -1                  ; End Flag
    end
